@@ -24,8 +24,6 @@
 # SOFTWARE.
 #
 
-__version_info__ = (0, 8, 0)
-__version__ = '.'.join(f'{x}' for x in __version_info__)
 
 from typing import NamedTuple, Any, Optional
 import logging
@@ -66,6 +64,8 @@ class Tag(NamedTuple):
     def __repr__(self):
         return f"{self.__class__.__name__}(tag={self.tag!r}, value={self.value!r}, type={self.type!r}, error={self.error!r})"
 
+
+from .cip_base import CIPDriver
 from .clx import LogixDriver
 from .const import CommonService, ClassCode, TagService, DataType, ConnectionManagerInstance, ConnectionManagerService
 from .bytes_ import Pack, Unpack
