@@ -24,16 +24,16 @@
 # SOFTWARE.
 #
 
-import logging
-from autologging import logged
 from reprlib import repr as _r
-from typing import Literal
+
+from autologging import logged
 
 from . import Packet, DataFormatType
 from . import (ResponsePacket, SendUnitDataResponsePacket, ReadTagServiceResponsePacket, RegisterSessionResponsePacket,
                UnRegisterSessionResponsePacket, ListIdentityResponsePacket, SendRRDataResponsePacket,
                MultiServiceResponsePacket, ReadTagFragmentedServiceResponsePacket, WriteTagServiceResponsePacket,
-               WriteTagFragmentedServiceResponsePacket, GenericUnconnectedResponsePacket, GenericConnectedResponsePacket)
+               WriteTagFragmentedServiceResponsePacket, GenericUnconnectedResponsePacket,
+               GenericConnectedResponsePacket)
 from .. import CommError, RequestError
 from ..bytes_ import Pack, print_bytes_msg
 from ..const import (EncapsulationCommand, INSUFFICIENT_PACKETS, DataItem, AddressItem, EXTENDED_SYMBOL, ELEMENT_TYPE,
@@ -89,8 +89,8 @@ class RequestPacket(Packet):
                 Pack.udint(self._plc.attribs['option']),  # Option UDINT
             ])
 
-        except Exception as e:
-            raise CommError(e)
+        except Exception as err:
+            raise CommError('Failed to build request header') from err
 
     def _build_common_packet_format(self, addr_data=None) -> bytes:
         addr_data = b'\x00\x00' if addr_data is None else Pack.uint(len(addr_data)) + addr_data
